﻿using System.Collections.Generic;
using Ozeki.VoIP;

namespace CallQueueStrategies.CallQueue
{
    class CallQueueExtension : IExtension
    {
        ICallManager callManager;
        List<string> members;
        object sync;
        CallQueueHandlerContainer callHandlers;
        CallQueueStrategy strategy;

        public CallQueueExtension(string extensionId, ICallManager callManager, List<string> members, CallQueueStrategy strategy)
        {
            sync = new object();
            callHandlers = new CallQueueHandlerContainer();
            ExtensionID = extensionId;
            this.callManager = callManager;
            this.strategy = strategy;
            this.members = members;
        }

        public string ExtensionID { get; private set; }

        public string Type
        {
            get { return "CallQueue"; }
        }

        public void OnCalled(IPBXCall call, IncomingCallParams callParams)
        {
            lock (sync)
            {
                var callQueueHandler = new CallQueueCallHandler(call, callManager, members, callHandlers, strategy);
                callQueueHandler.Completed += CallHandlerCompleted;
                callHandlers.Add(callQueueHandler);

                if (callHandlers.Count() == 1)
                {
                    callQueueHandler.Start();
                }
            }
        }

        public void OnMessageReceived(Message message)
        {
        }

        void CallHandlerCompleted(object sender, System.EventArgs e)
        {
            lock (sync)
            {
                var callHandler = (CallQueueCallHandler)sender;

                if (callHandlers.GetIndex(callHandler) == 0)
                {
                    
                    callHandlers.Remove(callHandler);

                    if (callHandlers.Count() >= 1)
                    {
                        var nextCallHandler = callHandlers.Next();
                        nextCallHandler.Start(callHandler.SelectedAgent);
                    }
                }
                else
                {
                    callHandlers.Remove(callHandler);
                }
            }
        }
    }
}
